<?php

/**
 * @file
 * Contains EC\EuropaSearch\Messages\Search\Filters\Clauses\TermsClause.
 */

namespace EC\EuropaSearch\Messages\Search\Filters\Clauses;

use EC\EuropaSearch\Messages\DocumentMetadata\BooleanMetadata;
use EC\EuropaSearch\Messages\DocumentMetadata\DateMetadata;
use EC\EuropaSearch\Messages\DocumentMetadata\FloatMetadata;
use EC\EuropaSearch\Messages\DocumentMetadata\IndexableMetadataInterface;
use EC\EuropaSearch\Messages\DocumentMetadata\IntegerMetadata;
use EC\EuropaSearch\Messages\DocumentMetadata\URLMetadata;
use Symfony\Component\Validator\Context\ExecutionContextInterface;
use Symfony\Component\Validator\Mapping\ClassMetadata;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * Class TermsClause.
 *
 * It defines a filter on a list of metadata values of scalar type.
 *
 * It does not support "NotIndexedMetadata" metadata type.
 *
 * @package EC\EuropaSearch\Messages\Search\Filters\Clauses
 */
class TermsClause extends AbstractClause
{

    /**
     * The values to use in the filter definition.
     *
     * @var array
     */
    private $testedValues;

    /**
     * FieldExist constructor.
     *
     * @param IndexableMetadataInterface $impliedMetadata
     *   The metadata pointed by the filters.
     */
    public function __construct(IndexableMetadataInterface $impliedMetadata)
    {
        $this->impliedMetadata = $impliedMetadata;
    }

    /**
     * {@inheritDoc}
     */
    public function getConverterIdentifier()
    {
        return self::CONVERTER_NAME_PREFIX.'terms';
    }

    /**
     * Gets the values to use in the filter definition.
     *
     * @return array
     * The values.
     */
    public function getTestedValues()
    {
        return $this->testedValues;
    }

    /**
     * Sets the values to use in the filter definition.
     *
     * @param array $testedValues
     *  The values.
     */
    public function setTestedValues($testedValues)
    {
        $this->testedValues = $testedValues;
    }

    /**
     * Loads constraints declarations for the validator process.
     *
     * @param ClassMetadata $metadata
     */
    public static function getConstraints(ClassMetadata $metadata)
    {

        $metadata->addPropertyConstraint('testedValues', new Assert\NotBlank());
        $metadata->addConstraint(new Assert\Callback('validate'));
    }

    /**
     * Special validator callback for valuesFilter.
     *
     * @param ExecutionContextInterface $context
     * @param mixed                     $payload
     */
    public function validate(ExecutionContextInterface $context, $payload)
    {

        $metadata = $this->getImpliedMetadata();
        switch (true) {
            case ($metadata instanceof DateMetadata):
                $this->validateDateRelatedFilter($context, $payload);
                break;

            case ($metadata instanceof FloatMetadata):
                $this->validateFloatRelatedFilter($context, $payload);
                break;

            case ($metadata instanceof IntegerMetadata):
                $this->validateIntRelatedFilter($context, $payload);
                break;

            case ($metadata instanceof BooleanMetadata):
                $this->validateBooleanRelatedFilter($context, $payload);
                break;

            case ($metadata instanceof URLMetadata):
                $this->validateURLRelatedFilter($context, $payload);
                break;
        }
    }

    /**
     * Special validator callback for valuesFilter related to a DateMetadata.
     *
     * @param ExecutionContextInterface $context
     * @param mixed                     $payload
     */
    private function validateDateRelatedFilter(ExecutionContextInterface $context, $payload)
    {

        foreach ($this->testedValues as $key => $testedValue) {
            if (!date_create($testedValue)) {
                $context->buildViolation('The tested value is not a valid date.')
                    ->atPath('testedValues['.$key.']')
                    ->addViolation();
            }
        }
    }

    /**
     * Special validator callback for valuesFilter related to a FloatMetadata.
     *
     * @param ExecutionContextInterface $context
     * @param mixed                     $payload
     */
    private function validateFloatRelatedFilter(ExecutionContextInterface $context, $payload)
    {

        foreach ($this->testedValues as $key => $testedValue) {
            if (!is_float($testedValue)) {
                $context->buildViolation('The tested value is not a valid float.')
                    ->atPath('testedValues['.$key.']')
                    ->addViolation();
            }
        }
    }

    /**
     * Special validator callback for valuesFilter related to a IntegerMetadata.
     *
     * @param ExecutionContextInterface $context
     * @param mixed                     $payload
     */
    private function validateIntRelatedFilter(ExecutionContextInterface $context, $payload)
    {

        foreach ($this->testedValues as $key => $testedValue) {
            if (!is_int($testedValue)) {
                $context->buildViolation('The tested value is not a valid integer.')
                    ->atPath('testedValues['.$key.']')
                    ->addViolation();
            }
        }
    }

    /**
     * Special validator callback for valuesFilter related to a BooleanMetadata.
     *
     * @param ExecutionContextInterface $context
     * @param mixed                     $payload
     */
    private function validateBooleanRelatedFilter(ExecutionContextInterface $context, $payload)
    {

        foreach ($this->testedValues as $key => $testedValue) {
            if (!is_bool($testedValue)) {
                $context->buildViolation('The tested value is not a valid boolean.')
                    ->atPath('testedValues['.$key.']')
                    ->addViolation();
            }
        }
    }

    /**
     * Special validator callback for valuesFilter related to a URLMetadata.
     *
     * @param ExecutionContextInterface $context
     * @param mixed                     $payload
     */
    private function validateURLRelatedFilter(ExecutionContextInterface $context, $payload)
    {

        foreach ($this->testedValues as $key => $testedValue) {
            $validator = new Assert\UrlValidator();
            $validator->initialize($context);
            // Re-inject the validator in the context in order to point on
            // the "testedValue" path.
            $validator = $context->getValidator()->inContext($context);
            $validator->atPath('testedValues['.$key.']')->validate($testedValue, new Assert\Url());
        }
    }
}
